/*
Version 1
envlogtest function - writes prog name & identifier to stderr and logs
environment details, tty in use, flags on fd's.
An example of envlogtest used as a function called by another program.

Copyright © 2014 C_Xperimental

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Contact C_Xperimental via the website <www.cxperimental.weebly.com>


 Configure logrotate for log file.
Before use: Create this file at this location: /etc/logrotate.d/envlogtest
With these contents shown below:
/var/log/envlogtest_log
{
        rotate 7
        daily
        dateext
        missingok
        notifempty
        nocompress
        copytruncate
}

Create the log file, set access permissions:
# touch  /var/log/envlogtest_log
# chmod a+rw  /var/log/envlogtest_log


Ensure all these includes below are included in the target program
*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <sys/types.h>


int
discover_flags (int file_des, int f_command)
{
  int flags_set;
  flags_set = fcntl (file_des, f_command);
  if (flags_set == -1) perror ("envlogtest flag_check failed");
  return (flags_set);
}

void
use_f_getfd (int file_des, FILE *log_out_file)
{
  int flags_set;
  flags_set = discover_flags (file_des, F_GETFD);
  if (flags_set & FD_CLOEXEC)
    {
      fprintf (log_out_file, "FD %d : FD_CLOEXEC set\n", file_des);
    }
  else
    {
      fprintf (log_out_file, "FD %d : FD_CLOEXEC not set\n", file_des);
    }
}

void
use_f_getfl (int file_des, FILE *log_out_file)
{
  int flags_set;
  flags_set = discover_flags (file_des, F_GETFL);
  if (flags_set & O_RDONLY)
    {
      fprintf (log_out_file, "FD %d : O_RDONLY set\n", file_des);
    }
  else
    {
      fprintf (log_out_file, "FD %d : O_RDONLY not set\n", file_des);
    }
  if (flags_set & O_WRONLY)
    {
      fprintf (log_out_file, "FD %d : O_WRONLY set\n", file_des);
    }
  else
    {
      fprintf (log_out_file, "FD %d : O_WRONLY not set\n", file_des);
    }
  if (flags_set & O_RDWR)
    {
      fprintf (log_out_file, "FD %d : O_RDWR set\n", file_des);
    }
  else
    {
      fprintf (log_out_file, "FD %d : O_RDWR not set\n", file_des);
    }
  if (flags_set & O_APPEND)
    {
      fprintf (log_out_file, "FD %d : O_APPEND set\n", file_des);
    }
  else
    {
      fprintf (log_out_file, "FD %d : O_APPEND not set\n", file_des);
    }
  if (flags_set & O_NONBLOCK)
    {
      fprintf (log_out_file, "FD %d : O_NONBLOCK set\n", file_des);
    }
  else
    {
      fprintf (log_out_file, "FD %d : O_NONBLOCK not set\n", file_des);
    }
  if (flags_set & O_ASYNC)
    {
      fprintf (log_out_file, "FD %d : O_ASYNC set\n", file_des);
    }
  else
    {
      fprintf (log_out_file, "FD %d : O_ASYNC not set\n", file_des);
    }
  if (flags_set & O_FSYNC)
    {
      fprintf (log_out_file, "FD %d : O_FSYNC set\n", file_des);
    }
  else
    {
      fprintf (log_out_file, "FD %d : O_FSYNC not set\n", file_des);
    }
}

void
envlogtest (char *ident)
{
  int count;
  char *terminal_device;
  char *log_file_name = "/var/log/envlogtest_log";
  time_t curtime;
  struct tm *loctime;
  int ngroups = getgroups (0, NULL);
  gid_t *groups;
  gid_t group_id;
  uid_t user_id;
  FILE *log_out_file;

  log_out_file = fopen (log_file_name, "a");
  if (log_out_file == NULL)
    {
      perror ("fopen failure");
      fprintf (stderr, "%s : envlogtest: %s could not be opened\n",
               ident, log_file_name);
    }

  flockfile (log_out_file);

  curtime = time (NULL);
  loctime = localtime (&curtime);
  fprintf (log_out_file, "\n\n\n%s\n", asctime (loctime));

    {
      fprintf (stderr, "%s : envlogtest-function\n", ident);
      fprintf (log_out_file, "%s : envlogtest-function\n", ident);
    }
  /* Process persona */
  user_id = getuid ();
  group_id = getgid ();
  fprintf (log_out_file, "Real user ID = %d, real group ID = %d\n",
           (int) user_id, (int) group_id);
  user_id = geteuid ();
  group_id = getegid ();
  fprintf (log_out_file, "Effective user ID = %d, effective group ID = %d\n",
           (int) user_id, (int) group_id);
  /* Supplementary group ID's */
  if (ngroups > 0)
    {
      groups = (gid_t *) malloc (ngroups * sizeof (gid_t));
      if (groups == NULL)
        {
          fprintf (stderr, "Memory allocation failure\n");
          fprintf (log_out_file, "Memory allocation failure\n");
          exit (EXIT_FAILURE);
        }
      if (ngroups > 50)
        {
          ngroups = 50;
          fprintf (stderr, "Truncated ngroups to 50\n");
          fprintf (log_out_file, "Truncated ngroups to 50\n");
        }
      count = getgroups (ngroups, groups);
      if (count < 0)
        {
          fprintf (stderr, "Error in getgroup ()\n");
          fprintf (log_out_file, "Error in getgroup ()\n");
        }
      fprintf (log_out_file, "Supplementary group ID's :\n");
      count--; /* convert for 0 start point of groups array*/
      while (count >= 0)
        {
          fprintf (log_out_file, "%d ", (int) groups[count]);
          count--;
        }
      fprintf (log_out_file, "\n");
    }
  else
    {
      fprintf (log_out_file, "No supplementary groups were found\n");
    }
  /* Environment variables that are set */
  count = 0;
  while (__environ[count] != NULL)
    {
      fprintf (log_out_file, "%s\n", __environ[count]);
      count++;
    }
  fprintf (log_out_file, "\n");

  terminal_device = ttyname (STDIN_FILENO);
  fprintf (log_out_file, "STDIN_FILENO  = %d : ttyname = %s\n",
           STDIN_FILENO, terminal_device);
  terminal_device = ttyname (STDOUT_FILENO);
  fprintf (log_out_file, "STDOUT_FILENO = %d : ttyname = %s\n",
           STDOUT_FILENO, terminal_device);
  terminal_device = ttyname (STDERR_FILENO);
  fprintf (log_out_file, "STDERR_FILENO = %d : ttyname = %s\n\n",
           STDERR_FILENO, terminal_device);

  use_f_getfd (STDIN_FILENO, log_out_file);
  use_f_getfl (STDIN_FILENO, log_out_file);
  use_f_getfd (STDOUT_FILENO, log_out_file);
  use_f_getfl (STDOUT_FILENO, log_out_file);
  use_f_getfd (STDERR_FILENO, log_out_file);
  use_f_getfl (STDERR_FILENO, log_out_file);

  fprintf (log_out_file, "\n\n\n");
  funlockfile (log_out_file);
  fclose (log_out_file);
}



/* "main" is only given here as an example of how envlogtest is called, in
 * practice envlogtest ("identifier"); would be called in the program of
 * interest.
 * But having this example main allows the envlogtest function to be tested:
 * Compile with:
 * $gcc -Wall -Wextra -o envlogfunction envlogfunction.c
 */


int
main ()
{
  puts ("Main function starting");

  /* Example call to envlogtest */
  envlogtest ("In main after puts");

  puts ("Main function finishing");
  return 0;
}
